#!/usr/bin/env python
#<examples/doc_model1.py>
from numpy import mean, sqrt, pi, exp, log, loadtxt
from scipy.stats import norm
from lmfit import  Model

import matplotlib.pyplot as plt

data = loadtxt('GaussAgeCrimeMales-Percentages.dat')
# data = loadtxt('GaussAgeCrimeFemales-Percentages.dat')
x = data[:, 0]
y = data[:, 1]

print ("**************************************************************")
print ("Curve Fitting Utility")
NumPoints = len(data)
print ("Number of Data Points = ", NumPoints)
YMean = mean(y)
print ("Mean Y = ", YMean)
print ("**************************************************************")

# NormalCDF with Constant

print ("**************************************************************")
print ("Normal CDF With Constant")
print ("**************************************************************")

def normalcdffunctionWC(x, p, m, b, c):
    "1-d normalcdffunctionWC: normalcdffunctionWC(x, p, m, b, c)"
    return (p/(1/c)*norm.cdf(m*x + b))

gmod = Model(normalcdffunctionWC)
result = gmod.fit(y, x=x, p = .03, m=.3, b=-5, c=1)

print(result.fit_report())

MDiffs = y-YMean
SStot = sum(MDiffs**2)
print ("SStot = ", SStot)

Ydiffs = y - result.best_fit
SSres = sum(Ydiffs**2)
print ("SSres = ", SSres)

CF = 1 - SSres/SStot
print ("Coefficient of Determination = ", CF)

# 2k + n ln(RSS) + C2.
k = 3
print ("Parameters = ", k)
AIC = 2*k + NumPoints * log(SSres)
print ("AIC = ", AIC)

# BIC = N * ln(RSS/N) + k * ln(N)
BIC = NumPoints * log(SSres/NumPoints) + k * log(NumPoints)
print ("BIC = ", BIC)

plt.plot(x, y,         'bo')
plt.plot(x, result.best_fit, 'r-')
plt.show()

# NormalCDF with Constant

print ("**************************************************************")
print ("Normal CDF Without Constant")
print ("**************************************************************")

def normalcdffunctionWOC(x, p, m, b):
    "1-d normalcdffunctionWOC: normalcdffunctionWOC(x, p, m, b)"
    return (p/norm.cdf(m*x + b))

gmod = Model(normalcdffunctionWOC)
result = gmod.fit(y, x=x, p = .03, m=-.02, b=-10)

print(result.fit_report())

MDiffs = y-YMean
SStot = sum(MDiffs**2)
print ("SStot = ", SStot)

Ydiffs = y - result.best_fit
SSres = sum(Ydiffs**2)
print ("SSres = ", SSres)

CF = 1 - SSres/SStot
print ("Coefficient of Determination = ", CF)

# 2k + n ln(RSS) + C2.
k = 2
print ("Parameters = ", k)

AIC = 2*k + NumPoints * log(SSres)
print ("AIC = ", AIC)

# BIC = N * ln(RSS/N) + k * ln(N)
BIC = NumPoints * log(SSres/NumPoints) + k * log(NumPoints)
print ("BIC = ", BIC)

plt.plot(x, y,         'bo')
plt.plot(x, result.best_fit, 'r-')
plt.show()

# Normal PDF

print ("**************************************************************")
print ("Normal PDF")
print ("**************************************************************")

def gaussianfunction(x, p, mean, sd):
    "1-d gaussianfunction: gaussianfunction(x, p, mean, sd)"
    return (p/(sqrt(2*pi)*sd)) * exp(-(x-mean)**2 /(2*sd**2))

gmod = Model(gaussianfunction)
result = gmod.fit(y, x=x, p=1, mean=0, sd=1)

print(result.fit_report())

MDiffs = y-YMean
SStot = sum(MDiffs**2)
print ("SStot = ", SStot)

Ydiffs = y - result.best_fit
SSres = sum(Ydiffs**2)
print ("SSres = ", SSres)

CF = 1 - SSres/SStot
print ("Coefficient of Determination = ", CF)

# 2k + n ln(RSS) + C2.
k = 2
print ("Parameters = ", k)

AIC = 2*k + NumPoints * log(SSres)
print ("AIC = ", AIC)

# BIC = N * ln(RSS/N) + k * ln(N)
BIC = NumPoints * log(SSres/NumPoints) + k * log(NumPoints)
print ("BIC = ", BIC)

plt.plot(x, y,         'bo')
plt.plot(x, result.best_fit, 'r-')
plt.show()

# Exponential Function

print ("**************************************************************")
print ("Exponential Function")
print ("**************************************************************")

def exponentialfunction(x, p, a, b, c):
    "1-d exponentialfunction: exponentialfunction(x, p, a, b, c)"
    return (p/(1/c) * exp(a*x + b))

gmod = Model(exponentialfunction)
result = gmod.fit(y, x=x, p=1, a=.3, b=0, c=1)

print(result.fit_report())

MDiffs = y-YMean
SStot = sum(MDiffs**2)
print ("SStot = ", SStot)

Ydiffs = y - result.best_fit
SSres = sum(Ydiffs**2)
print ("SSres = ", SSres)

CF = 1 - SSres/SStot
print ("Coefficient of Determination = ", CF)

# 2k + n ln(RSS) + C2.
k = 3
print ("Parameters = ", k)

AIC = 2*k + NumPoints * log(SSres)
print ("AIC = ", AIC)

# BIC = N * ln(RSS/N) + k * ln(N)
BIC = NumPoints * log(SSres/NumPoints) + k * log(NumPoints)
print ("BIC = ", BIC)

plt.plot(x, y,         'bo')
plt.plot(x, result.best_fit, 'r-')
plt.show()

# Cubic Function

print ("**************************************************************")
print ("Cubic Function")
print ("**************************************************************")

def cubicfunction(x, p, a, b, c, d):
   "1-d cubicfunction: cubicfunction(x, p, a, b, c, d)"
   return (p/a + b*x + c*x*x + d*x*x*x)

gmod = Model(cubicfunction)
result = gmod.fit(y, x=x, p=1, a=.3, b=1, c=1, d=1)

print(result.fit_report())

MDiffs = y-YMean
SStot = sum(MDiffs**2)
print ("SStot = ", SStot)

Ydiffs = y - result.best_fit
SSres = sum(Ydiffs**2)
print ("SSres = ", SSres)

CF = 1 - SSres/SStot
print ("Coefficient of Determination = ", CF)

# 2k + n ln(RSS) + C2.
k = 4
print ("Parameters = ", k)

AIC = 2*k + NumPoints * log(SSres)
print ("AIC = ", AIC)

# BIC = N * ln(RSS/N) + k * ln(N)
BIC = NumPoints * log(SSres/NumPoints) + k * log(NumPoints)
print ("BIC = ", BIC)

plt.plot(x, y,         'bo')
plt.plot(x, result.best_fit, 'r-')
plt.show()

# Quadratic Function

print ("**************************************************************")
print ("Quadratic Function")
print ("**************************************************************")
def quadraticfunction(x, p, a, b, c):
   "1-d quadraticfunction: quadraticfunction(x, p, a, b, c)"
   return (p/a + b*x + c*x*x)

gmod = Model(quadraticfunction)
result = gmod.fit(y, x=x, p=1, a=.3, b=1, c=1)

print(result.fit_report())

MDiffs = y-YMean
SStot = sum(MDiffs**2)
print ("SStot = ", SStot)

Ydiffs = y - result.best_fit
SSres = sum(Ydiffs**2)
print ("SSres = ", SSres)

CF = 1 - SSres/SStot
print ("Coefficient of Determination = ", CF)

# 2k + n ln(RSS) + C2.
k = 3
print ("Parameters = ", k)

AIC = 2*k + NumPoints * log(SSres)
print ("AIC = ", AIC)

# BIC = N * ln(RSS/N) + k * ln(N)
BIC = NumPoints * log(SSres/NumPoints) + k * log(NumPoints)
print ("BIC = ", BIC)

plt.plot(x, y,         'bo')
plt.plot(x, result.best_fit, 'r-')
plt.show()

# Linear Function

print ("**************************************************************")
print ("Linear Function")
print ("**************************************************************")

def linearfunction(x, p, slope, intercept):
   "1-d linearfunction: linearfunction(x, p, slope, intercept)"
   return (p/slope * x + intercept)

gmod = Model(linearfunction)
result = gmod.fit(y, x=x, p=1, slope=.5, intercept=-.02)

print(result.fit_report())

MDiffs = y-YMean
SStot = sum(MDiffs**2)
print ("SStot = ", SStot)

Ydiffs = y - result.best_fit
SSres = sum(Ydiffs**2)
print ("SSres = ", SSres)

CF = 1 - SSres/SStot
print ("Coefficient of Determination = ", CF)

# 2k + n ln(RSS) + C2.
k = 2
print ("Parameters = ", k)

AIC = 2*k + NumPoints * log(SSres)
print ("AIC = ", AIC)

# BIC = N * ln(RSS/N) + k * ln(N)
BIC = NumPoints * log(SSres/NumPoints) + k * log(NumPoints)
print ("BIC = ", BIC)

plt.plot(x, y,         'bo')
plt.plot(x, result.best_fit, 'r-')
plt.show()
#<end examples/doc_model1.py>